@@ -1,33 +1,21 @@
 /*
-Copyright (c) 2009-2013 Roger Light <roger@atchoo.org>
-All rights reserved.
+Copyright (c) 2009-2014 Roger Light <roger@atchoo.org>
 
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice,
-   this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. Neither the name of mosquitto nor the names of its
-   contributors may be used to endorse or promote products derived from
-   this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
-LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-POSSIBILITY OF SUCH DAMAGE.
+All rights reserved. This program and the accompanying materials
+are made available under the terms of the Eclipse Public License v1.0
+and Eclipse Distribution License v1.0 which accompany this distribution.
+ 
+The Eclipse Public License is available at
+   http://www.eclipse.org/legal/epl-v10.html
+and the Eclipse Distribution License is available at
+  http://www.eclipse.org/org/documents/edl-v10.php.
+ 
+Contributors:
+   Roger Light - initial implementation and documentation.
 */
 
 #include <assert.h>
+#include <time.h>
 
 #include <config.h>
 
@@ -37,7 +25,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "uthash.h"
 
-struct mosquitto *mqtt3_context_init(int sock)
+struct mosquitto *mqtt3_context_init(struct mosquitto_db *db, mosq_sock_t sock)
 {
 	struct mosquitto *context;
 	char address[1024];
@@ -70,20 +58,28 @@ struct mosquitto *mqtt3_context_init(int sock)
 	context->current_out_packet = NULL;
 
 	context->address = NULL;
-	if(!_mosquitto_socket_get_address(sock, address, 1024)){
-		context->address = _mosquitto_strdup(address);
-	}
-	if(!context->address && sock != -1){
-		/* getpeername and inet_ntop failed and not a bridge */
-		_mosquitto_free(context);
-		return NULL;
+	if((int)sock >= 0){
+		if(!_mosquitto_socket_get_address(sock, address, 1024)){
+			context->address = _mosquitto_strdup(address);
+		}
+		if(!context->address){
+			/* getpeername and inet_ntop failed and not a bridge */
+			_mosquitto_free(context);
+			return NULL;
+		}
 	}
 	context->bridge = NULL;
 	context->msgs = NULL;
+	context->last_msg = NULL;
+	context->msg_count = 0;
+	context->msg_count12 = 0;
 #ifdef WITH_TLS
 	context->ssl = NULL;
 #endif
 
+	if((int)context->sock >= 0){
+		HASH_ADD(hh_sock, db->contexts_by_sock, sock, sizeof(context->sock), context);
+	}
 	return context;
 }
 
@@ -97,6 +93,8 @@ void mqtt3_context_cleanup(struct mosquitto_db *db, struct mosquitto *context, b
 {
 	struct _mosquitto_packet *packet;
 	struct mosquitto_client_msg *msg, *next;
+	int i;
+
 	if(!context) return;
 
 	if(context->username){
@@ -109,46 +107,56 @@ void mqtt3_context_cleanup(struct mosquitto_db *db, struct mosquitto *context, b
 	}
 #ifdef WITH_BRIDGE
 	if(context->bridge){
-		if(context->bridge->username){
-			context->bridge->username = NULL;
+		for(i=0; i<db->bridge_count; i++){
+			if(db->bridges[i] == context){
+				db->bridges[i] = NULL;
+			}
 		}
-		if(context->bridge->password){
-			context->bridge->password = NULL;
+		if(context->bridge->local_clientid){
+			_mosquitto_free(context->bridge->local_clientid);
+			context->bridge->local_clientid = NULL;
+		}
+		if(context->bridge->remote_username){
+			context->bridge->remote_username = NULL;
+		}
+		if(context->bridge->remote_password){
+			context->bridge->remote_password = NULL;
+		}
+		if(context->bridge->local_username){
+			context->bridge->local_username = NULL;
+		}
+		if(context->bridge->local_password){
+			context->bridge->local_password = NULL;
+		}
+		if(context->bridge->local_clientid){
+			context->bridge->local_clientid = NULL;
 		}
 	}
 #endif
-	if(context->sock != -1){
-		if(context->listener){
-			context->listener->client_count--;
-			assert(context->listener->client_count >= 0);
-		}
-		_mosquitto_socket_close(context);
-		context->listener = NULL;
-	}
-	if(context->clean_session && db){
-		mqtt3_subs_clean_session(db, context, &db->subs);
-		mqtt3_db_messages_delete(context);
+	_mosquitto_socket_close(db, context);
+	if((do_free || context->clean_session) && db){
+		mqtt3_subs_clean_session(db, context);
+		mqtt3_db_messages_delete(db, context);
 	}
 	if(context->address){
 		_mosquitto_free(context->address);
 		context->address = NULL;
 	}
+
 	if(context->id){
-		// Remove the context's ID from the DB hash
-		struct _clientid_index_hash *find_cih;
-		HASH_FIND_STR(db->clientid_index_hash, context->id, find_cih);
-		if(find_cih){
-			// FIXME - internal level debug? _mosquitto_log_printf(NULL, MOSQ_LOG_INFO, "Found id for client \"%s\", their index was %d.", context->id, find_cih->db_context_index);
-			HASH_DEL(db->clientid_index_hash, find_cih);
-			_mosquitto_free(find_cih);
-		}else{
-			// FIXME - internal level debug? _mosquitto_log_printf(NULL, MOSQ_LOG_WARNING, "Unable to find id for client \"%s\".", context->id);
-		}
+		assert(db); /* db can only be NULL here if the client hasn't sent a
+					   CONNECT and hence wouldn't have an id. */
+
+		HASH_DELETE(hh_id, db->contexts_by_id, context);
 		_mosquitto_free(context->id);
 		context->id = NULL;
 	}
 	_mosquitto_packet_cleanup(&(context->in_packet));
-	_mosquitto_packet_cleanup(context->current_out_packet);
+	if(context->current_out_packet){
+		_mosquitto_packet_cleanup(context->current_out_packet);
+		_mosquitto_free(context->current_out_packet);
+		context->current_out_packet = NULL;
+	}
 	while(context->out_packet){
 		_mosquitto_packet_cleanup(context->out_packet);
 		packet = context->out_packet;
@@ -165,11 +173,12 @@ void mqtt3_context_cleanup(struct mosquitto_db *db, struct mosquitto *context, b
 		msg = context->msgs;
 		while(msg){
 			next = msg->next;
-			msg->store->ref_count--;
+			mosquitto__db_msg_store_deref(db, &msg->store);
 			_mosquitto_free(msg);
 			msg = next;
 		}
 		context->msgs = NULL;
+		context->last_msg = NULL;
 	}
 	if(do_free){
 		_mosquitto_free(context);
@@ -182,12 +191,37 @@ void mqtt3_context_disconnect(struct mosquitto_db *db, struct mosquitto *ctxt)
 		/* Unexpected disconnect, queue the client will. */
 		mqtt3_db_messages_easy_queue(db, ctxt, ctxt->will->topic, ctxt->will->qos, ctxt->will->payloadlen, ctxt->will->payload, ctxt->will->retain);
 	}
-	if(ctxt->listener){
-		ctxt->listener->client_count--;
-		assert(ctxt->listener->client_count >= 0);
-		ctxt->listener = NULL;
+	if(ctxt->will){
+		if(ctxt->will->topic) _mosquitto_free(ctxt->will->topic);
+		if(ctxt->will->payload) _mosquitto_free(ctxt->will->payload);
+		_mosquitto_free(ctxt->will);
+		ctxt->will = NULL;
 	}
-	ctxt->disconnect_t = mosquitto_time();
-	_mosquitto_socket_close(ctxt);
+	ctxt->disconnect_t = time(NULL);
+	_mosquitto_socket_close(db, ctxt);
+}
+
+void mosquitto__add_context_to_disused(struct mosquitto_db *db, struct mosquitto *context)
+{
+	if(db->ll_for_free){
+		context->for_free_next = db->ll_for_free;
+		db->ll_for_free = context;
+	}else{
+		db->ll_for_free = context;
+	}
+}
+
+void mosquitto__free_disused_contexts(struct mosquitto_db *db)
+{
+	struct mosquitto *context, *next;
+	assert(db);
+
+	context = db->ll_for_free;
+	while(context){
+		next = context->for_free_next;
+		mqtt3_context_cleanup(db, context, true);
+		context = next;
+	}
+	db->ll_for_free = NULL;
 }
 
