@@ -59,7 +59,37 @@ extern int g_clients_expired;
 static void loop_handle_errors(struct mosquitto_db *db, struct pollfd *pollfds);
 static void loop_handle_reads_writes(struct mosquitto_db *db, struct pollfd *pollfds);
 
-int mosquitto_main_loop(struct mosquitto_db *db, int *listensock, int listensock_count, int listener_max)
+#ifdef WITH_WEBSOCKETS
+static void temp__expire_websockets_clients(struct mosquitto_db *db)
+{
+	struct mosquitto *context, *ctxt_tmp;
+	static time_t last_check = 0;
+	time_t now = mosquitto_time();
+	char *id;
+
+	if(now - last_check > 60){
+		HASH_ITER(hh_id, db->contexts_by_id, context, ctxt_tmp){
+			if(context->wsi && context->sock != INVALID_SOCKET){
+				if(context->keepalive && now - context->last_msg_in > (time_t)(context->keepalive)*3/2){
+					if(db->config->connection_messages == true){
+						if(context->id){
+							id = context->id;
+						}else{
+							id = "<unknown>";
+						}
+						_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client %s has exceeded timeout, disconnecting.", id);
+					}
+					/* Client has exceeded keepalive*1.5 */
+					do_disconnect(db, context);
+				}
+			}
+		}
+		last_check = mosquitto_time();
+	}
+}
+#endif
+
+int mosquitto_main_loop(struct mosquitto_db *db, mosq_sock_t *listensock, int listensock_count, int listener_max)
 {
 #ifdef WITH_SYS_TREE
 	time_t start_time = mosquitto_time();
@@ -80,7 +110,7 @@ int mosquitto_main_loop(struct mosquitto_db *db, int *listensock, int listensock
 	int pollfd_count = 0;
 	int pollfd_index;
 #ifdef WITH_BRIDGE
-	int bridge_sock;
+	mosq_sock_t bridge_sock;
 	int rc;
 #endif
 	int context_count;
@@ -252,7 +282,7 @@ int mosquitto_main_loop(struct mosquitto_db *db, int *listensock, int listensock
 		now_time = time(NULL);
 		if(db->config->persistent_client_expiration > 0 && now_time > expiration_check_time){
 			HASH_ITER(hh_id, db->contexts_by_id, context, ctxt_tmp){
-				if(context->sock == -1 && context->clean_session == 0){
+				if(context->sock == INVALID_SOCKET && context->clean_session == 0){
 					/* This is a persistent client, check to see if the
 					 * last time it connected was longer than
 					 * persistent_client_expiration seconds ago. If so,
@@ -344,6 +374,9 @@ int mosquitto_main_loop(struct mosquitto_db *db, int *listensock, int listensock
 				libwebsocket_service(db->config->listeners[i].ws_context, 0);
 			}
 		}
+		if(db->config->have_websockets_listener){
+			temp__expire_websockets_clients(db);
+		}
 #endif
 	}
 
